use std::collections::HashMap;

use mongod::{assert_reference_of, Reference, Validate};

pub struct FlowInfo {
    pub name: String,
    pub depends: HashMap<String, i64>,
    pub next: Option<String>,
    pub produces: Option<Vec<String>>,
}

use mongod::{
    derive::{Model, Referencable},
    Model, Referencable, ToAPI,
};
use mongodb::bson::doc;
use serde::{Deserialize, Serialize};
use serde_json::json;

/// A production flow
#[derive(Debug, Clone, Serialize, Deserialize, Model, Referencable)]
pub struct Flow {
    /// ID
    pub _id: String,
    /// Tiemstamp when the flow was started
    pub started: i64,
    /// Timestamp when the flow was ended
    pub ended: i64,
    /// Kind of flow; ID of the describing JSON
    pub kind: String,
    /// The flow succedding this one
    pub next: Option<Reference>,
}

impl Validate for Flow {
    async fn validate(&self) -> Result<(), String> {
        if let Some(next) = &self.next {
            assert_reference_of!(next, Flow);
        }

        Ok(())
    }
}
