use mongod::{
    derive::{Model, Referencable},
    Model, ToAPI, Validate,
};
use mongodb::bson::doc;
use serde::{Deserialize, Serialize};
use serde_json::json;

/// A Storage Location
#[derive(Debug, Clone, Serialize, Deserialize, Model, Referencable)]
pub struct Location {
    /// UUID
    #[serde(default)]
    pub _id: String,
    /// Name
    pub name: String,
    /// Parent
    pub parent: Option<String>,
    /// Storage Conditions
    pub conditions: Option<StorageConditions>,
}

#[derive(Debug, Clone, Serialize, Deserialize, PartialEq, Eq)]
pub struct StorageConditions {
    /// Median temperature
    pub temperature: i64,
}

impl Validate for Location {
    async fn validate(&self) -> Result<(), String> {
        Ok(())
    }
}

impl ToAPI for Location {
    async fn api(&self) -> serde_json::Value {
        json!({
            "name": self.name,
            "parent": self.parent,
            "conditions": self.conditions
        })
    }
}

impl Location {
    pub async fn add(&mut self, id: &str) {
        self._id = id.to_string();
        self.insert_overwrite().await.unwrap();
    }
}
