use std::ops::Deref;

use json_store::JSONStore;
use location::Location;

use rocket::routes as route;
use rocket::{http::Method, launch};

mod config;
mod db;
mod flow;
mod item;
mod json_store;
mod location;
mod routes;
mod transaction;
mod variant;

// ░░░░░░░░░░▀▀▀██████▄▄▄░░░░░░░░░░
// ░░░░░░░░░░░░░░░░░▀▀▀████▄░░░░░░░
// ░░░░░░░░░░▄███████▀░░░▀███▄░░░░░
// ░░░░░░░░▄███████▀░░░░░░░▀███▄░░░
// ░░░░░░▄████████░░░░░░░░░░░███▄░░
// ░░░░░██████████▄░░░░░░░░░░░███▌░    ▒█▀▀█ █▀▀█ █▀▄▀█ █▀▄▀█ ▒█▀▀▄ ▒█▀▀█
// ░░░░░▀█████▀░▀███▄░░░░░░░░░▐███░    ▒█░░░ █░░█ █░▀░█ █░▀░█ ▒█░▒█ ▒█▀▀▄
// ░░░░░░░▀█▀░░░░░▀███▄░░░░░░░▐███░    ▒█▄▄█ ▀▀▀▀ ▀░░░▀ ▀░░░▀ ▒█▄▄▀ ▒█▄▄█
// ░░░░░░░░░░░░░░░░░▀███▄░░░░░███▌░
// ░░░░▄██▄░░░░░░░░░░░▀███▄░░▐███░░
// ░░▄██████▄░░░░░░░░░░░▀███▄███░░░
// ░█████▀▀████▄▄░░░░░░░░▄█████░░░░
// ░████▀░░░▀▀█████▄▄▄▄█████████▄░░
// ░░▀▀░░░░░░░░░▀▀██████▀▀░░░▀▀██░░

#[launch]
async fn rocket() -> _ {
    env_logger::init();

    let cors = rocket_cors::CorsOptions {
        allowed_origins: rocket_cors::AllowedOrigins::all(),
        allowed_methods: vec![Method::Get, Method::Post, Method::Options]
            .into_iter()
            .map(From::from)
            .collect(),
        allowed_headers: rocket_cors::AllowedHeaders::all(),
        allow_credentials: true,
        ..Default::default()
    }
    .to_cors()
    .expect("error creating CORS options");

    let itemdb = db::ItemDB::new("./itemdb").await;
    let locations: JSONStore<Location> = JSONStore::new("./locations");

    for location in locations.deref() {
        location.1.clone().add(location.0).await;
    }

    let config = config::get_config();

    rocket::build()
        .mount(
            "/",
            route![
                routes::item::get_items_route,
                routes::item::item_route,
                routes::item::item_variants_page,
                routes::item::supply_log_route,
                routes::item::demand_log_route,
                routes::item::supply_route,
                routes::item::demand_route,
                routes::item::transaction_route,
                routes::item::inventory_route,
                routes::item::inventory_route_variant,
                routes::item::variant_stat_route,
                routes::item::unique_field_route,
                routes::item::location_info,
                routes::item::locations_info,
                routes::item::locations_list
            ],
        )
        .manage(itemdb)
        .manage(locations)
        .manage(config)
        .attach(cors)
}
